@@ -1,5 +1,5 @@
 /* Functions to support bitset statistics.
-   Copyright (C) 2002 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2004 Free Software Foundation, Inc.
    Contributed by Michael Hayes (m.hayes@elec.canterbury.ac.nz).
 
 This program is free software; you can redistribute it and/or modify
@@ -14,20 +14,19 @@ GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
-Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.  */
 
 #ifndef _BITSET_STATS_H
 #define _BITSET_STATS_H
 
 #include "bbitset.h"
 
-extern int bitset_stats_enabled;
+extern bool bitset_stats_enabled;
 
-extern enum bitset_type bitset_stats_type_get PARAMS ((bitset));
+extern enum bitset_type bitset_stats_type_get (bitset);
 
-extern size_t bitset_stats_bytes PARAMS ((void));
+extern size_t bitset_stats_bytes (void);
 
-extern bitset bitset_stats_init PARAMS ((bitset, bitset_bindex,
-					 enum bitset_type));
+extern bitset bitset_stats_init (bitset, bitset_bindex, enum bitset_type);
 
 #endif
