@@ -1,5 +1,6 @@
 /* Grammar reduction for Bison.
-   Copyright (C) 2000, 2001, 2002  Free Software Foundation, Inc.
+
+   Copyright (C) 2000, 2001, 2002 Free Software Foundation, Inc.
 
    This file is part of Bison, the GNU Compiler Compiler.
 
@@ -15,16 +16,16 @@
 
    You should have received a copy of the GNU General Public License
    along with Bison; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.  */
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.  */
 
 #ifndef REDUCE_H_
 # define REDUCE_H_
 
-void reduce_grammar PARAMS ((void));
-void reduce_output PARAMS ((FILE *out));
-void reduce_free PARAMS ((void));
+void reduce_grammar (void);
+void reduce_output (FILE *out);
+void reduce_free (void);
 
-extern symbol_number_t nuseless_nonterminals;
-extern rule_number_t nuseless_productions;
+extern symbol_number nuseless_nonterminals;
+extern rule_number nuseless_productions;
 #endif /* !REDUCE_H_ */
